from enum import Enum


class Prognosegrundlage(str, Enum):
    """
    Prognosegrundlage describes the data used to perform a prognosis. This enum is not part of the official BO4E
    standard, but can be found the go and c# implementation:
    https://github.com/Hochfrequenz/go-bo4e/blob/main/enum/prognosegrundlage/prognosegrundlage.go
    https://github.com/Hochfrequenz/BO4E-dotnet/blob/main/BO4E/ENUM/Prognosegrundlage.cs
    """

    WERTE = "WERTE"
    PROFILE = "PROFILE"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.standorteigenschaften_gas import StandorteigenschaftenGas
from ..com.standorteigenschaften_strom import StandorteigenschaftenStrom
from ..enum.bo_typ import BoTyp


class Standorteigenschaften(BaseModel):
    """
    Modelliert die regionalen und spartenspezifischen Eigenschaften einer gegebenen Adresse.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Standorteigenschaften.svg" type="image/svg+xml"></object>

    .. HINT::
        `Standorteigenschaften JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Standorteigenschaften.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.STANDORTEIGENSCHAFTEN, alias="boTyp")]
    eigenschaften_gas: Annotated[StandorteigenschaftenGas | None, Field(None, alias="eigenschaftenGas")]
    eigenschaften_strom: Annotated[
        list[StandorteigenschaftenStrom] | None, Field(None, alias="eigenschaftenStrom", title="Eigenschaftenstrom")
    ]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated


class Sigmoidparameter(BaseModel):
    """
    Die Sigmoid-Funktion, beispielsweise zur Berechnung eines Leistungspreises hat die Form:
    LP=A/(1+(P/B)^C)+D

    .. raw:: html

        <object data="../_static/images/bo4e/com/Sigmoidparameter.svg" type="image/svg+xml"></object>

    .. HINT::
        `Sigmoidparameter JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Sigmoidparameter.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    a: Annotated[float | str | None, Field(None, alias="A", title="A")]
    b: Annotated[float | str | None, Field(None, alias="B", title="B")]
    c: Annotated[float | str | None, Field(None, alias="C", title="C")]
    d: Annotated[float | str | None, Field(None, alias="D", title="D")]
    id: Annotated[str, Field(alias="_id", title=" Id")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from .tarifpreisstaffel_pro_ort import TarifpreisstaffelProOrt


class TarifpreispositionProOrt(BaseModel):
    """
    Mit dieser Komponente können Tarifpreise verschiedener Typen abgebildet werden

    .. raw:: html

        <object data="../_static/images/bo4e/com/TarifpreispositionProOrt.svg" type="image/svg+xml"></object>

    .. HINT::
        `TarifpreispositionProOrt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/TarifpreispositionProOrt.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    netznr: Annotated[str | None, Field(None, title="Netznr")]
    ort: Annotated[str | None, Field(None, title="Ort")]
    postleitzahl: Annotated[str | None, Field(None, title="Postleitzahl")]
    preisstaffeln: Annotated[list[TarifpreisstaffelProOrt] | None, Field(None, title="Preisstaffeln")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.mengeneinheit import Mengeneinheit
from ..enum.preistyp import Preistyp
from ..enum.waehrungseinheit import Waehrungseinheit
from .regionale_preisstaffel import RegionalePreisstaffel


class RegionaleTarifpreisposition(BaseModel):
    """
    Mit dieser Komponente können Tarifpreise verschiedener Typen im Zusammenhang mit regionalen Gültigkeiten abgebildet
    werden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/RegionaleTarifpreisposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `RegionaleTarifpreisposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/RegionaleTarifpreisposition.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    bezugseinheit: Mengeneinheit | None = None
    einheit: Waehrungseinheit | None = None
    mengeneinheitstaffel: Mengeneinheit | None = None
    preisstaffeln: Annotated[list[RegionalePreisstaffel] | None, Field(None, title="Preisstaffeln")]
    preistyp: Preistyp | None = None



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.betrag import Betrag
from ..com.externe_referenz import ExterneReferenz
from ..com.kostenblock import Kostenblock
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.kostenklasse import Kostenklasse


class Kosten(BaseModel):
    """
    Dieses BO wird zur Übertagung von hierarchischen Kostenstrukturen verwendet.
    Die Kosten werden dabei in Kostenblöcke und diese wiederum in Kostenpositionen strukturiert.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Kosten.svg" type="image/svg+xml"></object>

    .. HINT::
        `Kosten JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Kosten.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.KOSTEN, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    gueltigkeit: Zeitraum | None = None
    kostenbloecke: Annotated[list[Kostenblock] | None, Field(None, title="Kostenbloecke")]
    kostenklasse: Kostenklasse | None = None
    summe_kosten: Annotated[list[Betrag] | None, Field(None, alias="summeKosten", title="Summekosten")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]



from enum import Enum


class Bemessungsgroesse(str, Enum):
    """
    Zur Abbildung von Messgrössen und zur Verwendung in energiewirtschaftlichen Berechnungen.
    """

    WIRKARBEIT_EL = "WIRKARBEIT_EL"
    LEISTUNG_EL = "LEISTUNG_EL"
    BLINDARBEIT_KAP = "BLINDARBEIT_KAP"
    BLINDARBEIT_IND = "BLINDARBEIT_IND"
    BLINDLEISTUNG_KAP = "BLINDLEISTUNG_KAP"
    BLINDLEISTUNG_IND = "BLINDLEISTUNG_IND"
    WIRKARBEIT_TH = "WIRKARBEIT_TH"
    LEISTUNG_TH = "LEISTUNG_TH"
    VOLUMEN = "VOLUMEN"
    VOLUMENSTROM = "VOLUMENSTROM"
    BENUTZUNGSDAUER = "BENUTZUNGSDAUER"
    ANZAHL = "ANZAHL"



from enum import Enum


class Tariftyp(str, Enum):
    """
    Zur Differenzierung von Grund/Ersatzversorgungstarifen und sonstigen angebotenen Tarifen.
    """

    GRUND_ERSATZVERSORGUNG = "GRUND_ERSATZVERSORGUNG"
    GRUNDVERSORGUNG = "GRUNDVERSORGUNG"
    ERSATZVERSORGUNG = "ERSATZVERSORGUNG"
    SONDERTARIF = "SONDERTARIF"



from pydantic import AwareDatetime, BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..bo.vertrag import Vertrag
from ..enum.kontaktart import Kontaktart
from .adresse import Adresse


class VertragskontoCBA(BaseModel):
    """
    Models a CBA (child billing account) which directly relates to a single contract. It contains information about
    locks and billing dates. But in the first place, CBAs will be grouped together by the address in their contracts.
    For each group of CBAs with a common address there will be created an MBA (master billing
    account) to support that the invoices for the CBAs can be bundled into a single invoice for the MBA.
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    ouid: Annotated[int, Field(title="Ouid")]
    vertrags_adresse: Annotated[Adresse, Field(alias="vertragsAdresse")]
    vertragskontonummer: Annotated[str, Field(title="Vertragskontonummer")]
    rechnungsstellung: Kontaktart
    vertrag: Vertrag
    erstellungsdatum: Annotated[AwareDatetime, Field(title="Erstellungsdatum")]
    rechnungsdatum_start: Annotated[AwareDatetime, Field(alias="rechnungsdatumStart", title="Rechnungsdatumstart")]
    rechnungsdatum_naechstes: Annotated[
        AwareDatetime, Field(alias="rechnungsdatumNaechstes", title="Rechnungsdatumnaechstes")
    ]



from enum import Enum


class Ausschreibungstyp(str, Enum):
    """
    Aufzählung für die Typisierung von Ausschreibungen.
    """

    PRIVATRECHTLICH = "PRIVATRECHTLICH"
    OEFFENTLICHRECHTLICH = "OEFFENTLICHRECHTLICH"
    EUROPAWEIT = "EUROPAWEIT"



from enum import Enum


class Vertragsart(str, Enum):
    """
    Aufzählung der Vertragsarten.
    """

    ENERGIELIEFERVERTRAG = "ENERGIELIEFERVERTRAG"
    NETZNUTZUNGSVERTRAG = "NETZNUTZUNGSVERTRAG"
    BILANZIERUNGSVERTRAG = "BILANZIERUNGSVERTRAG"
    MESSSTELLENBETRIEBSVERTRAG = "MESSSTELLENBETRIEBSVERTRAG"
    BUENDELVERTRAG = "BUENDELVERTRAG"



from pydantic import AwareDatetime, BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.bdew_artikelnummer import BDEWArtikelnummer
from ..enum.zeiteinheit import Zeiteinheit
from .betrag import Betrag
from .menge import Menge
from .preis import Preis
from .steuerbetrag import Steuerbetrag


class Rechnungsposition(BaseModel):
    """
    Über Rechnungspositionen werden Rechnungen strukturiert.
    In einem Rechnungsteil wird jeweils eine in sich geschlossene Leistung abgerechnet.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Rechnungsposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Rechnungsposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Rechnungsposition.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    artikel_id: Annotated[str | None, Field(None, alias="artikelId", title="Artikelid")]
    artikelnummer: BDEWArtikelnummer | None = None
    einzelpreis: Preis | None = None
    lieferung_bis: Annotated[AwareDatetime | None, Field(None, alias="lieferungBis", title="Lieferungbis")]
    lieferung_von: Annotated[AwareDatetime | None, Field(None, alias="lieferungVon", title="Lieferungvon")]
    lokations_id: Annotated[str | None, Field(None, alias="lokationsId", title="Lokationsid")]
    positions_menge: Annotated[Menge | None, Field(None, alias="positionsMenge")]
    positionsnummer: Annotated[int | None, Field(None, title="Positionsnummer")]
    positionstext: Annotated[str | None, Field(None, title="Positionstext")]
    teilrabatt_netto: Annotated[Betrag | None, Field(None, alias="teilrabattNetto")]
    teilsumme_netto: Annotated[Betrag | None, Field(None, alias="teilsummeNetto")]
    teilsumme_steuer: Annotated[Steuerbetrag | None, Field(None, alias="teilsummeSteuer")]
    zeitbezogene_menge: Annotated[Menge | None, Field(None, alias="zeitbezogeneMenge")]
    zeiteinheit: Zeiteinheit | None = None



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.kontaktart import Kontaktart
from .adresse import Adresse
from .vertragskonto_cba import VertragskontoCBA


class VertragskontoMBA(BaseModel):
    """
    Models an MBA (master billing account). Its main purpose is to bundle CBAs together having the same address in
    their related contracts. This feature supports a single invoice for all CBAs instead of several
    invoices for each.
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    ouid: Annotated[int, Field(title="Ouid")]
    vertrags_adresse: Annotated[Adresse, Field(alias="vertragsAdresse")]
    vertragskontonummer: Annotated[str, Field(title="Vertragskontonummer")]
    rechnungsstellung: Kontaktart
    cbas: Annotated[list[VertragskontoCBA], Field(title="Cbas")]



from enum import Enum


class Messwertstatuszusatz(str, Enum):
    """
    Aufzählung von zusätzlichen Informationen zum Status, beispielsweise in Lastgängen oder Zählwerkständen.
    """

    Z84_LEERSTAND = "Z84_LEERSTAND"
    Z85_REALERZAEHLERUEBERLAUFGEPRUEFT = "Z85_REALERZAEHLERUEBERLAUFGEPRUEFT"
    Z86_PLAUSIBELWGKONTROLLABLESUNG = "Z86_PLAUSIBELWGKONTROLLABLESUNG"
    Z87_PLAUSIBELWGKUNDENHINWEIS = "Z87_PLAUSIBELWGKUNDENHINWEIS"
    ZC3_AUSTAUSCHDESERSATZWERTES = "ZC3_AUSTAUSCHDESERSATZWERTES"
    Z88_VERGLEICHSMESSUNG_GEEICHT_ = "Z88_VERGLEICHSMESSUNG(GEEICHT)"
    Z89_VERGLEICHSMESSUNG_NICHTGEEICHT_ = "Z89_VERGLEICHSMESSUNG(NICHTGEEICHT)"
    Z90_MESSWERTNACHBILDUNGAUSGEEICHTENWERTEN = "Z90_MESSWERTNACHBILDUNGAUSGEEICHTENWERTEN"
    Z91_MESSWERTNACHBILDUNGAUSNICHTGEEICHTENWERTEN = "Z91_MESSWERTNACHBILDUNGAUSNICHTGEEICHTENWERTEN"
    Z92_INTERPOLATION = "Z92_INTERPOLATION"
    Z93_HALTEWERT = "Z93_HALTEWERT"
    Z94_BILANZIERUNGNETZABSCHNITT = "Z94_BILANZIERUNGNETZABSCHNITT"
    Z95_HISTORISCHEMESSWERTE = "Z95_HISTORISCHEMESSWERTE"
    ZJ2_STATISTISCHEMETHODE = "ZJ2_STATISTISCHEMETHODE"
    Z74_KEINZUGANG = "Z74_KEINZUGANG"
    Z75_KOMMUNIKATIONSSTOERUNG = "Z75_KOMMUNIKATIONSSTOERUNG"
    Z76_NETZAUSFALL = "Z76_NETZAUSFALL"
    Z77_SPANNUNGSAUSFALL = "Z77_SPANNUNGSAUSFALL"
    Z78_GERAETEWECHSEL = "Z78_GERAETEWECHSEL"
    Z79_KALIBRIERUNG = "Z79_KALIBRIERUNG"
    Z80_GERAETARBEITETAUSSERHALBDERBETRIEBSBEDINGUNGEN = "Z80_GERAETARBEITETAUSSERHALBDERBETRIEBSBEDINGUNGEN"
    Z81_MESSEINRICHTUNGGESTOERT_DEFEKT = "Z81_MESSEINRICHTUNGGESTOERT_DEFEKT"
    Z82_UNSICHERHEITMESSUNG = "Z82_UNSICHERHEITMESSUNG"
    Z98_BERUECKSICHTIGUNGSTOERMENGENZAEHLWERK = "Z98_BERUECKSICHTIGUNGSTOERMENGENZAEHLWERK"
    Z99_MENGENUMWERTUNGUNVOLLSTAENDIG = "Z99_MENGENUMWERTUNGUNVOLLSTAENDIG"
    ZA0_UHRZEITGESTELLT_SYNCHRONISATION = "ZA0_UHRZEITGESTELLT_SYNCHRONISATION"
    ZA1_MESSWERTUNPLAUSIBEL = "ZA1_MESSWERTUNPLAUSIBEL"
    ZC2_TARIFSCHALTGERAETDEFEKT = "ZC2_TARIFSCHALTGERAETDEFEKT"
    ZC4_IMPULSWERTIGKEITNICHTAUSREICHEND = "ZC4_IMPULSWERTIGKEITNICHTAUSREICHEND"
    ZA3_FALSCHERWANDLERFAKTOR = "ZA3_FALSCHERWANDLERFAKTOR"
    ZA4_FEHLERHAFTEABLESUNG = "ZA4_FEHLERHAFTEABLESUNG"
    ZA5_AENDERUNGDERBERECHNUNG = "ZA5_AENDERUNGDERBERECHNUNG"
    ZA6_UMBAUDERMESSLOKATION = "ZA6_UMBAUDERMESSLOKATION"
    ZA7_DATENBEARBEITUNGSFEHLER = "ZA7_DATENBEARBEITUNGSFEHLER"
    ZA8_BRENNWERTKORREKTUR = "ZA8_BRENNWERTKORREKTUR"
    ZA9_Z_ZAHL_KORREKTUR = "ZA9_Z-ZAHL-KORREKTUR"
    ZB0_STOERUNG_DEFEKTMESSEINRICHTUNG = "ZB0_STOERUNG_DEFEKTMESSEINRICHTUNG"
    ZB9_AENDERUNGTARIFSCHALTZEITEN = "ZB9_AENDERUNGTARIFSCHALTZEITEN"
    ZG3_UMSTELLUNGGASQUALITAET = "ZG3_UMSTELLUNGGASQUALITAET"



"""
Contains the dataset for the customer loader.
It also contains the validation logic for the customer loader dataset.
"""

from ibims.bo4e import Adresse, Bankverbindung, Geschaeftspartner, VertragskontoMBA
from ibims.datasets import DataSetBaseModel


class TripicaCustomerLoaderDataSet(DataSetBaseModel):
    """
    This is a bo4e data set that consists of
    * a GeschaeftspartnerErweitert
    * a Adresse
    * a List of Bankverbindung
    * a List of Vertragskonto
    In the context of this package is may be used to create Tripica Customer Data.
    """

    powercloud_customer_id: str  #: the powercloud customer id that was used to create this data set
    # this allows us to track a customer from its source up the target
    # https://github.com/Hochfrequenz/bo4e_migration_framework/blob/6f091b76ff4e7a72bb0ff6ecb46a8477d35b5bf8/src/bomf/model/__init__.py#L70

    geschaeftspartner_erw: Geschaeftspartner
    """
    The following attributes need to be filled for this DataSet:
    - name1 (Surname)
    - name2 (Firstname)
    - name3 (Title e.g. Dr.)
    - erstellungsdatum
    - geburtstag
    - telefonnummer_privat
    - telefonnummer_mobil
    - telefonnummer_geschaeft
    - kontaktweg
    and within each kontaktweg:
    - list[Kontaktart (ANSCHREIBEN/E_MAIL/TELEFONAT)]
    - externe_referenzen
    and within each externe referenz:
    - customerID
    """

    liefer_adressen: dict[str, Adresse]  # [contract_id, Adresse]
    """
    The following attributes need to be filled for this DataSet:
    - postleitzahl
    - ort
    - strasse
    - hausnummer
    - landescode
    """

    rechnungs_adressen: dict[str, Adresse]  # [contract_id, Adresse]
    """
    The following attributes need to be filled for this DataSet:
    - postleitzahl
    - ort
    - strasse
    - hausnummer
    - landescode
    """

    banks: dict[str, Bankverbindung]  # [contract_id, Bankverbindung]
    """
    Each Bankverbindung in banks requires the following attributes to be filled for this DataSet:
    - iban
    - bic
    - bankname
    - kontoinhaber
    - sepa_info
    and within each sepa_info:
    - sepa_id
    - gueltig_seit
    """

    vertragskonten_mbas: list[VertragskontoMBA]  # MBA
    """
    Each VertragskontoMBA (master billing account) needs the following Attributes:
    - ouid
    - vertragskontonummer
    - vertrags_adresse
    # -billing_cycle might be necessary, if so, it still needs to be implemented.
    # https://github.com/Hochfrequenz/powercloud_to_lynqtech_data_model_transformation/issues/271
    - cbas
    And within each CBA:
    - this contains all child billing accounts grouped together by the vertragsadresse. I.e. an MBA and all its
    CBAs share a common `vertrags_adresse`.
    """

    def get_id(self) -> str:
        return self.powercloud_customer_id



"""
Contains the dataset for the document loader.
It also contains the validation logic for the document loader dataset.
"""

from typing import Optional

from ibims.bo4e import Dokument, File, Geschaeftspartner, Rechnung, Vertrag
from ibims.datasets import DataSetBaseModel


class TripicaDocumentLoaderDataSet(DataSetBaseModel):

    """
    This is a bo4e data set that consists of
    * a GeschaeftspartnerErweitert
    * a Vertrag
    * a document
    * optionally a Rechnung
    * optionally a File
    In the context of this package is may be used to create Tripica Document Data.
    """

    doc_id: str
    """
    a unique id identifying a document in the tripica data loader set (typically a filename)
    """

    geschaeftspartner_erw: Geschaeftspartner
    """
    The following attribute needs to be filled for this DataSet:
    - externe_referenzen
    especially the customerID
    """

    vertrag: Vertrag
    """
    Each Vertrag needs the following Attribute:
    - vertragsnummer
    """

    rechnung: Optional[Rechnung] = None
    """
    The following attribute needs to be filled for this DataSet:
    - rechnungsnummer
    """

    dokument: Dokument
    """
    A reference to a document
    """

    file: Optional[File] = None
    """
    A reference to a file object
    """



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.geraetetyp import Geraetetyp


class Hardware(BaseModel):
    """
    Abbildung einer abrechenbaren Hardware

    .. raw:: html

        <object data="../_static/images/bo4e/com/Hardware.svg" type="image/svg+xml"></object>

    .. HINT::
        `Hardware JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Hardware.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    bezeichnung: Annotated[str | None, Field(None, title="Bezeichnung")]
    geraetetyp: Geraetetyp | None = None



from pydantic import AwareDatetime, BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.zaehlwerk import Zaehlwerk
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.messwerterfassung import Messwerterfassung
from ..enum.sparte import Sparte
from ..enum.tarifart import Tarifart
from ..enum.zaehlerauspraegung import Zaehlerauspraegung
from ..enum.zaehlertyp import Zaehlertyp
from .geschaeftspartner import Geschaeftspartner


class Zaehler(BaseModel):
    """
    Object containing information about a meter/"Zaehler".

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Zaehler.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zaehler JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Zaehler.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.ZAEHLER, alias="boTyp")]
    eichung_bis: Annotated[AwareDatetime | None, Field(None, alias="eichungBis", title="Eichungbis")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    letzte_eichung: Annotated[AwareDatetime | None, Field(None, alias="letzteEichung", title="Letzteeichung")]
    sparte: Sparte | None = None
    tarifart: Tarifart | None = None
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    zaehlerauspraegung: Zaehlerauspraegung | None = None
    zaehlerhersteller: Geschaeftspartner | None = None
    zaehlerkonstante: Annotated[float | str | None, Field(None, title="Zaehlerkonstante")]
    zaehlernummer: Annotated[str | None, Field(None, title="Zaehlernummer")]
    zaehlertyp: Zaehlertyp | None = None
    zaehlwerke: Annotated[list[Zaehlwerk] | None, Field(None, title="Zaehlwerke")]
    messwerterfassung: Messwerterfassung
    nachstes_ablesedatum: Annotated[
        AwareDatetime | None, Field(None, alias="nachstesAblesedatum", title="Nachstesablesedatum")
    ]
    aktiver_zeitraum: Annotated[Zeitraum | None, Field(None, alias="aktiverZeitraum")]



from enum import Enum


class ArithmetischeOperation(str, Enum):
    """
    Mit dieser Aufzählung können arithmetische Operationen festgelegt werden.
    """

    ADDITION = "ADDITION"
    SUBTRAKTION = "SUBTRAKTION"
    MULTIPLIKATION = "MULTIPLIKATION"
    DIVISION = "DIVISION"



from pydantic import AwareDatetime, BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.messwertstatus import Messwertstatus
from ..enum.messwertstatuszusatz import Messwertstatuszusatz


class Zeitreihenwert(BaseModel):
    """
    Abbildung eines Zeitreihenwertes bestehend aus Zeitraum, Wert und Statusinformationen.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zeitreihenwert.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitreihenwert JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Zeitreihenwert.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    datum_uhrzeit_bis: Annotated[AwareDatetime | None, Field(None, alias="datumUhrzeitBis", title="Datumuhrzeitbis")]
    datum_uhrzeit_von: Annotated[AwareDatetime | None, Field(None, alias="datumUhrzeitVon", title="Datumuhrzeitvon")]
    status: Messwertstatus | None = None
    statuszusatz: Messwertstatuszusatz | None = None
    wert: Annotated[float | str | None, Field(None, title="Wert")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.preismodell import Preismodell
from ..enum.rechnungslegung import Rechnungslegung
from ..enum.sparte import Sparte
from ..enum.vertragsform import Vertragsform
from .ausschreibungsdetail import Ausschreibungsdetail
from .menge import Menge
from .zeitraum import Zeitraum


class Ausschreibungslos(BaseModel):
    """
    Eine Komponente zur Abbildung einzelner Lose einer Ausschreibung

    .. raw:: html

        <object data="../_static/images/bo4e/com/Ausschreibungslos.svg" type="image/svg+xml"></object>

    .. HINT::
        `Ausschreibungslos JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Ausschreibungslos.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    anzahl_lieferstellen: Annotated[int | None, Field(None, alias="anzahlLieferstellen", title="Anzahllieferstellen")]
    bemerkung: Annotated[str | None, Field(None, title="Bemerkung")]
    betreut_durch: Annotated[str | None, Field(None, alias="betreutDurch", title="Betreutdurch")]
    bezeichnung: Annotated[str | None, Field(None, title="Bezeichnung")]
    energieart: Sparte | None = None
    gesamt_menge: Annotated[Menge | None, Field(None, alias="gesamtMenge")]
    lieferstellen: Annotated[list[Ausschreibungsdetail] | None, Field(None, title="Lieferstellen")]
    lieferzeitraum: Zeitraum | None = None
    losnummer: Annotated[str | None, Field(None, title="Losnummer")]
    preismodell: Preismodell | None = None
    wiederholungsintervall: Zeitraum | None = None
    wunsch_kuendingungsfrist: Annotated[Zeitraum | None, Field(None, alias="wunschKuendingungsfrist")]
    wunsch_maximalmenge: Annotated[Menge | None, Field(None, alias="wunschMaximalmenge")]
    wunsch_mindestmenge: Annotated[Menge | None, Field(None, alias="wunschMindestmenge")]
    wunsch_rechnungslegung: Annotated[Rechnungslegung | None, Field(None, alias="wunschRechnungslegung")]
    wunsch_vertragsform: Annotated[Vertragsform | None, Field(None, alias="wunschVertragsform")]
    wunsch_zahlungsziel: Annotated[Zeitraum | None, Field(None, alias="wunschZahlungsziel")]



from enum import Enum


class Tarifregionskriterium(str, Enum):
    """
    Mit diesen Kriterien können regionale Bereiche definiert werden.
    """

    NETZ_NUMMER = "NETZ_NUMMER"
    POSTLEITZAHL = "POSTLEITZAHL"
    ORT = "ORT"
    GRUNDVERSORGER_NUMMER = "GRUNDVERSORGER_NUMMER"
    REGION = "REGION"



"""
Contains the dataset for the invoice loader
"""
from typing import Optional

from ibims.bo4e import Bilanzierung, Marktlokation, Marktteilnehmer, Rechnung, Vertrag
from ibims.datasets import DataSetBaseModel
from ibims.targetmodels import TransaktionsdatenInvoices


class InvoiceLoaderDataSet(DataSetBaseModel):
    """
    This is a bo4e dat set that consists out of
    * a Marktlokation
    * a RechnungErweitert
    * a Marktteilnhemer
    * a Bilanzierung (which is only relevant for a memi)
    * a Vertrag (which is only relevant for a memi)
    """

    marktlokation: Marktlokation
    """
    The following attributes need to be filled for this DataSet:
    - lokationsadresse
    - marktlokations_id
    - sparte
    """

    rechnung_erweitert: Rechnung
    """
    The following attributes need to be filled for this Dataset:
    - rechungstyp
    - rechungsnummer
    - rechungsdatum
    - faelligkeitsdatum
    - rechungsperiode
    - gesamtnetto
    - gesamtsteuer
    - gesamtbrutto, this value should be negative for self issued invoices
    - zuzahlen, this value should be negative for self issued invoices
    - rechnungspositionen, the values of the fields teilsumme_steuer and teilsumme_netto should be negative for
    self-issued invoices
    - steuerbetrag, the values of the fields basiswert and steuerwert should be negative for self issued invoices
    - ist_selbstausgestellt,
    - ist_reverse_charge
    """

    marktteilnehmer: Marktteilnehmer
    """
    The following attributes need to be filled for this DataSet:
    - name1 (organization)
    - partneradresse
    - rollencodenummer
    - rollencodetyp
    - ansprechoartner
    - sparte
    """

    bilanzierung: Optional[Bilanzierung] = None
    """
    The following attributes need to be filled for this DataSet:
    - bilanzierungsbeginn
    - bilanzierungsende
    """

    vertrag: Optional[Vertrag] = None
    """
    The following attributes need to be filled for this DataSet:
    - vertragskonditionen (only the field netznutzungsabbrechung is needed)
    """

    transaktionsdaten: TransaktionsdatenInvoices
    """
    The following attributes need to be filled for this DataSet:
    - sparte
    - pruefidentifikator
    - lieferrichtung
    - duplikat
    - referenznummer
    - datenaustauschreferenz
    - absender
    - empfaenger
    - nachrichtendatum
    - nachrichten_referenznummer
    """



from pydantic import AwareDatetime, BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.betrag import Betrag
from ..com.externe_referenz import ExterneReferenz
from ..com.rechnungsposition import Rechnungsposition
from ..com.steuerbetrag import Steuerbetrag
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.rechnungsstatus import Rechnungsstatus
from ..enum.rechnungstyp import Rechnungstyp
from .geschaeftspartner import Geschaeftspartner


class Rechnung(BaseModel):
    """
    Modell für die Abbildung von Rechnungen im Kontext der Energiewirtschaft;
    Ausgehend von diesem Basismodell werden weitere spezifische Formen abgeleitet.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Rechnung.svg" type="image/svg+xml"></object>

    .. HINT::
        `Rechnung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Rechnung.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.RECHNUNG, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    faelligkeitsdatum: Annotated[AwareDatetime | None, Field(None, title="Faelligkeitsdatum")]
    gesamtbrutto: Betrag | None = None
    gesamtnetto: Betrag | None = None
    gesamtsteuer: Betrag | None = None
    original_rechnungsnummer: Annotated[
        str | None, Field(None, alias="originalRechnungsnummer", title="Originalrechnungsnummer")
    ]
    rabatt_brutto: Annotated[Betrag | None, Field(None, alias="rabattBrutto")]
    rechnungsdatum: Annotated[AwareDatetime | None, Field(None, title="Rechnungsdatum")]
    rechnungsempfaenger: Geschaeftspartner | None = None
    rechnungsersteller: Geschaeftspartner | None = None
    rechnungsnummer: Annotated[str | None, Field(None, title="Rechnungsnummer")]
    rechnungsperiode: Zeitraum | None = None
    rechnungspositionen: Annotated[list[Rechnungsposition] | None, Field(None, title="Rechnungspositionen")]
    rechnungsstatus: Rechnungsstatus | None = None
    rechnungstitel: Annotated[str | None, Field(None, title="Rechnungstitel")]
    rechnungstyp: Rechnungstyp | None = None
    steuerbetraege: Annotated[list[Steuerbetrag] | None, Field(None, title="Steuerbetraege")]
    storno: Annotated[bool | None, Field(None, title="Storno")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    vorausgezahlt: Betrag | None = None
    zuzahlen: Betrag | None = None
    ist_selbstausgestellt: Annotated[
        bool | None, Field(None, alias="istSelbstausgestellt", title="Istselbstausgestellt")
    ]
    ist_reverse_charge: Annotated[bool | None, Field(None, alias="istReverseCharge", title="Istreversecharge")]



from enum import Enum


class Zaehlerauspraegung(str, Enum):
    """
    Gibt an, ob es sich um einen Einrichtungs- oder Zweirichtungszähler handelt.
    """

    EINRICHTUNGSZAEHLER = "EINRICHTUNGSZAEHLER"
    ZWEIRICHTUNGSZAEHLER = "ZWEIRICHTUNGSZAEHLER"



from enum import Enum


class Anrede(str, Enum):
    """
    Übersicht möglicher Anreden, z.B. eines Geschäftspartners.
    """

    HERR = "HERR"
    FRAU = "FRAU"
    EHELEUTE = "EHELEUTE"
    FIRMA = "FIRMA"
    INDIVIDUELL = "INDIVIDUELL"



"""
Contains classes that describe the boneycomb data structure and provides the base Transaktionsdaten class
"""
from typing import Literal, Optional

from pydantic import BaseModel

from ibims.bo4e import Sparte


class Transaktionsdaten(BaseModel):
    """
    This class collects the data in transaktionsdaten
    """

    migration_id: Optional[str] = None
    import_fuer_storno_adhoc: Literal["true", "false"]
    sparte: Sparte
    pruefidentifikator: str
    datenaustauschreferenz: str
    nachrichtendatum: str
    nachrichten_referenznummer: str
    absender: str
    empfaenger: str


QuantitiesStatus = Literal["CANCELLED", "DECISION", "IGNORED", "OBJECTED", "RECEIVED", "VALID"]


class TransaktionsdatenQuantities(Transaktionsdaten):
    """
    This class adds additional data to the transaktionsdaten, which is needed for a energy amount
    """

    migration_id: Optional[str] = None
    dokumentennummer: str
    kategorie: str
    nachrichtenfunktion: str
    typ: str
    datumsformat: str
    status: QuantitiesStatus


InvoiceManagerInvoiceStatus = Literal["received", "ignored", "declined", "cancelled", "accepted", "manual_decision"]


class TransaktionsdatenInvoices(Transaktionsdaten):
    """
    This class adds additional data to the transaktionsdaten, which is needed for an invoice
    """

    lieferrichtung: Optional[str] = None
    referenznummer: Optional[str] = None
    duplikat: Literal["true", "false"]
    status: InvoiceManagerInvoiceStatus
    # the boneycombs transaktionsdaten are (string,string) key value pairs
    # hence this is not a real boolean but two possible literal strings.



from enum import Enum


class Themengebiet(str, Enum):
    """
    Über dieses ENUM kann eine thematische Zuordnung, beispielsweise eines Ansprechpartners, vorgenommen werden.
    """

    ALLGEMEINER_INFORMATIONSAUSTAUSCH = "ALLGEMEINER_INFORMATIONSAUSTAUSCH"
    AN_UND_ABMELDUNG = "AN_UND_ABMELDUNG"
    ANSPRECHPARTNER_ALLGEMEIN = "ANSPRECHPARTNER_ALLGEMEIN"
    ANSPRECHPARTNER_BDEW_DVGW = "ANSPRECHPARTNER_BDEW_DVGW"
    ANSPRECHPARTNER_IT_TECHNIK = "ANSPRECHPARTNER_IT_TECHNIK"
    BILANZIERUNG = "BILANZIERUNG"
    BILANZKREISKOORDINATOR = "BILANZKREISKOORDINATOR"
    BILANZKREISVERANTWORTLICHER = "BILANZKREISVERANTWORTLICHER"
    DATENFORMATE_ZERTIFIKATE_VERSCHLUESSELUNGEN = "DATENFORMATE_ZERTIFIKATE_VERSCHLUESSELUNGEN"
    DEBITORENMANAGEMENT = "DEBITORENMANAGEMENT"
    DEMAND_SIDE_MANAGEMENT = "DEMAND_SIDE_MANAGEMENT"
    EDI_VEREINBARUNG = "EDI_VEREINBARUNG"
    EDIFACT = "EDIFACT"
    ENERGIEDATENMANAGEMENT = "ENERGIEDATENMANAGEMENT"
    FAHRPLANMANAGEMENT = "FAHRPLANMANAGEMENT"
    ALOCAT = "ALOCAT"
    APERAK = "APERAK"
    CONTRL = "CONTRL"
    INVOIC = "INVOIC"
    MSCONS = "MSCONS"
    ORDERS = "ORDERS"
    ORDERSP = "ORDERSP"
    REMADV = "REMADV"
    UTILMD = "UTILMD"
    GABI = "GABI"
    GELI = "GELI"
    GERAETERUECKGABE = "GERAETERUECKGABE"
    GERAETEWECHSEL = "GERAETEWECHSEL"
    GPKE = "GPKE"
    INBETRIEBNAHME = "INBETRIEBNAHME"
    KAPAZITAETSMANAGEMENT = "KAPAZITAETSMANAGEMENT"
    KLAERFAELLE = "KLAERFAELLE"
    LASTGAENGE_RLM = "LASTGAENGE_RLM"
    LIEFERANTENRAHMENVERTRAG = "LIEFERANTENRAHMENVERTRAG"
    LIEFERANTENWECHSEL = "LIEFERANTENWECHSEL"
    MABIS = "MABIS"
    MAHNWESEN = "MAHNWESEN"
    MARKTGEBIETSVERANTWORTLICHER = "MARKTGEBIETSVERANTWORTLICHER"
    MARKTKOMMUNIKATION = "MARKTKOMMUNIKATION"
    MEHR_MINDERMENGEN = "MEHR_MINDERMENGEN"
    MSB_MDL = "MSB_MDL"
    NETZABRECHNUNG = "NETZABRECHNUNG"
    NETZENTGELTE = "NETZENTGELTE"
    NETZMANAGEMENT = "NETZMANAGEMENT"
    RECHT = "RECHT"
    REGULIERUNGSMANAGEMENT = "REGULIERUNGSMANAGEMENT"
    REKLAMATIONEN = "REKLAMATIONEN"
    SPERREN_ENTSPERREN_INKASSO = "SPERREN_ENTSPERREN_INKASSO"
    STAMMDATEN = "STAMMDATEN"
    STOERUNGSFAELLE = "STOERUNGSFAELLE"
    TECHNISCHE_FRAGEN = "TECHNISCHE_FRAGEN"
    UMSTELLUNG_INVOIC = "UMSTELLUNG_INVOIC"
    VERSCHLUESSELUNG_SIGNATUR = "VERSCHLUESSELUNG_SIGNATUR"
    VERTRAGSMANAGEMENT = "VERTRAGSMANAGEMENT"
    VERTRIEB = "VERTRIEB"
    WIM = "WIM"
    ZAEHLERSTAENDE_SLP = "ZAEHLERSTAENDE_SLP"
    ZAHLUNGSVERKEHR = "ZAHLUNGSVERKEHR"
    ZUORDNUNGSVEREINBARUNG = "ZUORDNUNGSVEREINBARUNG"



from enum import Enum


class Tarifkalkulationsmethode(str, Enum):
    """
    Auflistung der verschiedenen Berechnungsmethoden für ein Preisblatt.
    """

    KEINE = "KEINE"
    STAFFELN = "STAFFELN"
    ZONEN = "ZONEN"
    BESTABRECHNUNG_STAFFEL = "BESTABRECHNUNG_STAFFEL"
    PAKETPREIS = "PAKETPREIS"



from enum import Enum


class AblesendeRolle(str, Enum):
    """
    Eine (Markt)Rolle, die Verbräuche abliest.
    """

    VNB = "VNB"
    ENDKUNDE = "ENDKUNDE"
    VORIGER_LIEFERANT = "VORIGER_LIEFERANT"
    MSB = "MSB"
    SYSTEM = "SYSTEM"



from pydantic import AwareDatetime, BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.unterschrift import Unterschrift
from ..com.vertragskonditionen import Vertragskonditionen
from ..enum.bo_typ import BoTyp
from ..enum.sparte import Sparte
from ..enum.vertragsart import Vertragsart
from ..enum.vertragsstatus import Vertragsstatus
from .geschaeftspartner import Geschaeftspartner
from .vertrag import Vertrag


class Buendelvertrag(BaseModel):
    """
    Abbildung eines Bündelvertrags.
    Es handelt sich hierbei um eine Liste von Einzelverträgen, die in einem Vertragsobjekt gebündelt sind.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Buendelvertrag.svg" type="image/svg+xml"></object>

    .. HINT::
        `Buendelvertrag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Buendelvertrag.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    beschreibung: Annotated[str | None, Field(None, title="Beschreibung")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.BUENDELVERTRAG, alias="boTyp")]
    einzelvertraege: Annotated[list[Vertrag] | None, Field(None, title="Einzelvertraege")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    sparte: Sparte | None = None
    unterzeichnervp1: Annotated[list[Unterschrift] | None, Field(None, title="Unterzeichnervp1")]
    unterzeichnervp2: Annotated[list[Unterschrift] | None, Field(None, title="Unterzeichnervp2")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    vertragsart: Vertragsart | None = None
    vertragsbeginn: Annotated[AwareDatetime | None, Field(None, title="Vertragsbeginn")]
    vertragsende: Annotated[AwareDatetime | None, Field(None, title="Vertragsende")]
    vertragskonditionen: Annotated[list[Vertragskonditionen] | None, Field(None, title="Vertragskonditionen")]
    vertragsnummer: Annotated[str | None, Field(None, title="Vertragsnummer")]
    vertragspartner1: Geschaeftspartner | None = None
    vertragspartner2: Geschaeftspartner | None = None
    vertragsstatus: Vertragsstatus | None = None



from pydantic import AwareDatetime, BaseModel, ConfigDict, Field
from typing_extensions import Annotated


class SepaInfo(BaseModel):
    """
    This class includes details about the sepa mandates.
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    sepa_id: Annotated[str, Field(alias="sepaId", title="Sepaid")]
    sepa_zahler: Annotated[bool, Field(alias="sepaZahler", title="Sepazahler")]
    creditor_identifier: Annotated[str | None, Field(None, alias="creditorIdentifier", title="Creditoridentifier")]
    gueltig_seit: Annotated[AwareDatetime | None, Field(None, alias="gueltigSeit", title="Gueltigseit")]



from enum import Enum


class Mengeneinheit(str, Enum):
    """
    Einheit: Messgrößen, die per Messung oder Vorgabe ermittelt werden können.
    """

    W = "W"
    WH = "WH"
    KW = "KW"
    KWH = "KWH"
    KVARH = "KVARH"
    MW = "MW"
    MWH = "MWH"
    STUECK = "STUECK"
    KUBIKMETER = "KUBIKMETER"
    STUNDE = "STUNDE"
    TAG = "TAG"
    MONAT = "MONAT"
    JAHR = "JAHR"
    PROZENT = "PROZENT"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.adresse import Adresse
from ..com.externe_referenz import ExterneReferenz
from ..com.rufnummer import Rufnummer
from ..com.zustaendigkeit import Zustaendigkeit
from ..enum.anrede import Anrede
from ..enum.bo_typ import BoTyp
from ..enum.titel import Titel
from .geschaeftspartner import Geschaeftspartner


class Ansprechpartner(BaseModel):
    """
    Object containing information about a Ansprechpartner

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Ansprechpartner.svg" type="image/svg+xml"></object>

    .. HINT::
        `Ansprechpartner JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Ansprechpartner.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    adresse: Adresse | None = None
    anrede: Anrede | None = None
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.ANSPRECHPARTNER, alias="boTyp")]
    e_mail_adresse: Annotated[str | None, Field(None, alias="eMailAdresse", title="Emailadresse")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    geschaeftspartner: Geschaeftspartner | None = None
    individuelle_anrede: Annotated[str | None, Field(None, alias="individuelleAnrede", title="Individuelleanrede")]
    kommentar: Annotated[str | None, Field(None, title="Kommentar")]
    nachname: Annotated[str | None, Field(None, title="Nachname")]
    rufnummer: Rufnummer | None = None
    titel: Titel | None = None
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    vorname: Annotated[str | None, Field(None, title="Vorname")]
    zustaendigkeit: Zustaendigkeit | None = None



from pydantic import BaseModel, Field
from typing_extensions import Annotated


class File(BaseModel):
    """
    This class represents a file that is stored in the database.
    """

    file_name_for_docstore: Annotated[str | None, Field(None, title="File Name For Docstore")]
    folder_name_for_docstore: Annotated[str | None, Field(None, title="Folder Name For Docstore")]
    file: Annotated[bytes, Field(title="File")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated


class ExterneReferenz(BaseModel):
    """
    Viele Datenobjekte weisen in unterschiedlichen Systemen eine eindeutige ID (Kundennummer, GP-Nummer etc.) auf.
    Beim Austausch von Datenobjekten zwischen verschiedenen Systemen ist es daher hilfreich,
    sich die eindeutigen IDs der anzubindenden Systeme zu merken.

    .. raw:: html

        <object data="../_static/images/bo4e/com/ExterneReferenz.svg" type="image/svg+xml"></object>

    .. HINT::
        `ExterneReferenz JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/ExterneReferenz.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    ex_ref_name: Annotated[str | None, Field(None, alias="exRefName", title="Exrefname")]
    ex_ref_wert: Annotated[str | None, Field(None, alias="exRefWert", title="Exrefwert")]



from enum import Enum


class Netzebene(str, Enum):
    """
    Auflistung möglicher Netzebenen innerhalb der Energiearten Strom und Gas.
    """

    NSP = "NSP"
    MSP = "MSP"
    HSP = "HSP"
    HSS = "HSS"
    MSP_NSP_UMSP = "MSP_NSP_UMSP"
    HSP_MSP_UMSP = "HSP_MSP_UMSP"
    HSS_HSP_UMSP = "HSS_HSP_UMSP"
    HD = "HD"
    MD = "MD"
    ND = "ND"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..enum.bo_typ import BoTyp


class Kampagne(BaseModel):
    """
    A "Kampagne"/campaign models which marketing activities led customers to a product/tariff.
    """

    model_config = ConfigDict(
        extra="allow",
    )
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.GESCHAEFTSOBJEKT, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    id: Annotated[str, Field(title="Id")]
    name: Annotated[str, Field(title="Name")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.gueltigkeitstyp import Gueltigkeitstyp
from .kriterium_wert import KriteriumWert


class RegionaleGueltigkeit(BaseModel):
    """
    Mit dieser Komponente können regionale Gültigkeiten, z.B. für Tarife, Zu- und Abschläge und Preise definiert werden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/RegionaleGueltigkeit.svg" type="image/svg+xml"></object>

    .. HINT::
        `RegionaleGueltigkeit JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/RegionaleGueltigkeit.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    gueltigkeitstyp: Gueltigkeitstyp | None = None
    kriteriums_werte: Annotated[
        list[KriteriumWert] | None, Field(None, alias="kriteriumsWerte", title="Kriteriumswerte")
    ]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.kontaktart import Kontaktart
from .adresse import Adresse


class Vertragskonto(BaseModel):
    """
    Bundle common attributes of MBA and CBA.
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    ouid: Annotated[int, Field(title="Ouid")]
    vertrags_adresse: Annotated[Adresse, Field(alias="vertragsAdresse")]
    vertragskontonummer: Annotated[str, Field(title="Vertragskontonummer")]
    rechnungsstellung: Kontaktart



"""
BO4E vunknown - Generated Python implementation of the BO4E standard

BO4E is a standard for the exchange of business objects in the energy industry.
All our software used to generate this BO4E-implementation is open-source and released under the Apache-2.0 license.

The BO4E version can be queried using `bo4e.__version__`.
"""

__all__ = [
    "Angebot",
    "Ansprechpartner",
    "Ausschreibung",
    "Bilanzierung",
    "Buendelvertrag",
    "Dokument",
    "Energiemenge",
    "File",
    "Fremdkosten",
    "Geschaeftsobjekt",
    "Geschaeftspartner",
    "Hinweis",
    "Kampagne",
    "Kosten",
    "Lastgang",
    "LastgangKompakt",
    "Marktlokation",
    "Marktteilnehmer",
    "Messlokation",
    "Netznutzungsrechnung",
    "Preisblatt",
    "PreisblattDienstleistung",
    "PreisblattHardware",
    "PreisblattKonzessionsabgabe",
    "PreisblattMessung",
    "PreisblattNetznutzung",
    "Rechnung",
    "Region",
    "Regionaltarif",
    "Standorteigenschaften",
    "Tarif",
    "Tarifinfo",
    "Tarifkosten",
    "Tarifpreisblatt",
    "Vertrag",
    "Zaehler",
    "ZaehlerGas",
    "Zeitreihe",
    "Adresse",
    "Angebotsposition",
    "Angebotsteil",
    "Angebotsvariante",
    "AufAbschlag",
    "AufAbschlagProOrt",
    "AufAbschlagRegional",
    "AufAbschlagstaffelProOrt",
    "Ausschreibungsdetail",
    "Ausschreibungslos",
    "Bankverbindung",
    "Betrag",
    "COM",
    "ConcessionFee",
    "Dienstleistung",
    "Energieherkunft",
    "Energiemix",
    "ExterneReferenz",
    "Fremdkostenblock",
    "Fremdkostenposition",
    "Geokoordinaten",
    "Geraet",
    "Geraeteeigenschaften",
    "Hardware",
    "Katasteradresse",
    "Kostenblock",
    "Kostenposition",
    "KriteriumWert",
    "Lastprofil",
    "MarktgebietInfo",
    "Menge",
    "Messlokationszuordnung",
    "PositionsAufAbschlag",
    "Preis",
    "Preisgarantie",
    "Preisposition",
    "Preisstaffel",
    "Rechnungsposition",
    "RegionaleGueltigkeit",
    "RegionalePreisgarantie",
    "RegionalePreisstaffel",
    "RegionalerAufAbschlag",
    "RegionaleTarifpreisposition",
    "Regionskriterium",
    "Rufnummer",
    "SepaInfo",
    "Sigmoidparameter",
    "StandorteigenschaftenGas",
    "StandorteigenschaftenStrom",
    "Steuerbetrag",
    "Tagesvektor",
    "Tarifberechnungsparameter",
    "Tarifeinschraenkung",
    "Tarifpreis",
    "Tarifpreisposition",
    "TarifpreispositionProOrt",
    "TarifpreisstaffelProOrt",
    "Unterschrift",
    "Verbrauch",
    "Vertragskonditionen",
    "Vertragskonto",
    "VertragskontoCBA",
    "VertragskontoMBA",
    "Vertragsteil",
    "Zaehlpunkt",
    "Zaehlwerk",
    "Zeitintervall",
    "Zeitraum",
    "Zeitreihenwert",
    "Zeitreihenwertkompakt",
    "Zustaendigkeit",
    "Abgabeart",
    "AblesendeRolle",
    "Ablesungsstatus",
    "Aggregationsverantwortung",
    "Angebotsstatus",
    "Anrede",
    "ArithmetischeOperation",
    "ArtikelId",
    "AufAbschlagstyp",
    "AufAbschlagsziel",
    "Ausschreibungsportal",
    "Ausschreibungsstatus",
    "Ausschreibungstyp",
    "BDEWArtikelnummer",
    "Bemessungsgroesse",
    "Bilanzierungsmethode",
    "BoTyp",
    "Dienstleistungstyp",
    "Energierichtung",
    "Erzeugungsart",
    "Gasqualitaet",
    "Gebiettyp",
    "Geraetemerkmal",
    "Geraetetyp",
    "Geschaeftspartnerrolle",
    "Gueltigkeitstyp",
    "HinweisThema",
    "Kalkulationsmethode",
    "Kontaktart",
    "Kostenklasse",
    "Kundengruppe",
    "KundengruppeKA",
    "Kundentyp",
    "Landescode",
    "Leistungstyp",
    "Lokationstyp",
    "Marktgebiet",
    "Marktrolle",
    "Medium",
    "Mengeneinheit",
    "Mengenoperator",
    "Messart",
    "Messgroesse",
    "Messpreistyp",
    "MesstechnischeEinordnung",
    "Messwerterfassung",
    "Messwertstatus",
    "Messwertstatuszusatz",
    "Netzebene",
    "NNRechnungsart",
    "NNRechnungstyp",
    "Oekolabel",
    "Oekozertifikat",
    "Preisgarantietyp",
    "Preismodell",
    "Preisstatus",
    "Preistyp",
    "Profiltyp",
    "Prognosegrundlage",
    "Rechnungslegung",
    "Rechnungsstatus",
    "Rechnungstyp",
    "Regelzone",
    "Regionskriteriumtyp",
    "Rollencodetyp",
    "Rufnummernart",
    "Sparte",
    "Steuerkennzeichen",
    "Tarifart",
    "Tarifkalkulationsmethode",
    "Tarifmerkmal",
    "Tarifregionskriterium",
    "Tariftyp",
    "Tarifzeit",
    "Themengebiet",
    "Titel",
    "Variant",
    "Verbrauchsart",
    "Vertragsart",
    "Vertragsform",
    "Vertragsstatus",
    "Voraussetzungen",
    "Waehrungscode",
    "Waehrungseinheit",
    "Wertermittlungsverfahren",
    "Zaehlerauspraegung",
    "Zaehlertyp",
    "Zeiteinheit",
    "__version__",
]

from .__version__ import __version__
from .bo.angebot import Angebot
from .bo.ansprechpartner import Ansprechpartner
from .bo.ausschreibung import Ausschreibung
from .bo.bilanzierung import Bilanzierung
from .bo.buendelvertrag import Buendelvertrag
from .bo.dokument import Dokument
from .bo.energiemenge import Energiemenge
from .bo.file import File
from .bo.fremdkosten import Fremdkosten
from .bo.geschaeftsobjekt import Geschaeftsobjekt
from .bo.geschaeftspartner import Geschaeftspartner
from .bo.hinweis import Hinweis
from .bo.kampagne import Kampagne
from .bo.kosten import Kosten
from .bo.lastgang import Lastgang
from .bo.lastgang_kompakt import LastgangKompakt
from .bo.marktlokation import Marktlokation
from .bo.marktteilnehmer import Marktteilnehmer
from .bo.messlokation import Messlokation
from .bo.netznutzungsrechnung import Netznutzungsrechnung
from .bo.preisblatt import Preisblatt
from .bo.preisblatt_dienstleistung import PreisblattDienstleistung
from .bo.preisblatt_hardware import PreisblattHardware
from .bo.preisblatt_konzessionsabgabe import PreisblattKonzessionsabgabe
from .bo.preisblatt_messung import PreisblattMessung
from .bo.preisblatt_netznutzung import PreisblattNetznutzung
from .bo.rechnung import Rechnung
from .bo.region import Region
from .bo.regionaltarif import Regionaltarif
from .bo.standorteigenschaften import Standorteigenschaften
from .bo.tarif import Tarif
from .bo.tarifinfo import Tarifinfo
from .bo.tarifkosten import Tarifkosten
from .bo.tarifpreisblatt import Tarifpreisblatt
from .bo.vertrag import Vertrag
from .bo.zaehler import Zaehler
from .bo.zaehler_gas import ZaehlerGas
from .bo.zeitreihe import Zeitreihe
from .com.adresse import Adresse
from .com.angebotsposition import Angebotsposition
from .com.angebotsteil import Angebotsteil
from .com.angebotsvariante import Angebotsvariante
from .com.auf_abschlag import AufAbschlag
from .com.auf_abschlag_pro_ort import AufAbschlagProOrt
from .com.auf_abschlag_regional import AufAbschlagRegional
from .com.auf_abschlagstaffel_pro_ort import AufAbschlagstaffelProOrt
from .com.ausschreibungsdetail import Ausschreibungsdetail
from .com.ausschreibungslos import Ausschreibungslos
from .com.bankverbindung import Bankverbindung
from .com.betrag import Betrag
from .com.com import COM
from .com.concession_fee import ConcessionFee
from .com.dienstleistung import Dienstleistung
from .com.energieherkunft import Energieherkunft
from .com.energiemix import Energiemix
from .com.externe_referenz import ExterneReferenz
from .com.fremdkostenblock import Fremdkostenblock
from .com.fremdkostenposition import Fremdkostenposition
from .com.geokoordinaten import Geokoordinaten
from .com.geraet import Geraet
from .com.geraeteeigenschaften import Geraeteeigenschaften
from .com.hardware import Hardware
from .com.katasteradresse import Katasteradresse
from .com.kostenblock import Kostenblock
from .com.kostenposition import Kostenposition
from .com.kriterium_wert import KriteriumWert
from .com.lastprofil import Lastprofil
from .com.marktgebiet_info import MarktgebietInfo
from .com.menge import Menge
from .com.messlokationszuordnung import Messlokationszuordnung
from .com.positions_auf_abschlag import PositionsAufAbschlag
from .com.preis import Preis
from .com.preisgarantie import Preisgarantie
from .com.preisposition import Preisposition
from .com.preisstaffel import Preisstaffel
from .com.rechnungsposition import Rechnungsposition
from .com.regionale_gueltigkeit import RegionaleGueltigkeit
from .com.regionale_preisgarantie import RegionalePreisgarantie
from .com.regionale_preisstaffel import RegionalePreisstaffel
from .com.regionale_tarifpreisposition import RegionaleTarifpreisposition
from .com.regionaler_auf_abschlag import RegionalerAufAbschlag
from .com.regionskriterium import Regionskriterium
from .com.rufnummer import Rufnummer
from .com.sepa_info import SepaInfo
from .com.sigmoidparameter import Sigmoidparameter
from .com.standorteigenschaften_gas import StandorteigenschaftenGas
from .com.standorteigenschaften_strom import StandorteigenschaftenStrom
from .com.steuerbetrag import Steuerbetrag
from .com.tagesvektor import Tagesvektor
from .com.tarifberechnungsparameter import Tarifberechnungsparameter
from .com.tarifeinschraenkung import Tarifeinschraenkung
from .com.tarifpreis import Tarifpreis
from .com.tarifpreisposition import Tarifpreisposition
from .com.tarifpreisposition_pro_ort import TarifpreispositionProOrt
from .com.tarifpreisstaffel_pro_ort import TarifpreisstaffelProOrt
from .com.unterschrift import Unterschrift
from .com.verbrauch import Verbrauch
from .com.vertragskonditionen import Vertragskonditionen
from .com.vertragskonto import Vertragskonto
from .com.vertragskonto_cba import VertragskontoCBA
from .com.vertragskonto_mba import VertragskontoMBA
from .com.vertragsteil import Vertragsteil
from .com.zaehlpunkt import Zaehlpunkt
from .com.zaehlwerk import Zaehlwerk
from .com.zeitintervall import Zeitintervall
from .com.zeitraum import Zeitraum
from .com.zeitreihenwert import Zeitreihenwert
from .com.zeitreihenwertkompakt import Zeitreihenwertkompakt
from .com.zustaendigkeit import Zustaendigkeit
from .enum.abgabeart import Abgabeart
from .enum.ablesende_rolle import AblesendeRolle
from .enum.ablesungsstatus import Ablesungsstatus
from .enum.aggregationsverantwortung import Aggregationsverantwortung
from .enum.angebotsstatus import Angebotsstatus
from .enum.anrede import Anrede
from .enum.arithmetische_operation import ArithmetischeOperation
from .enum.artikel_id import ArtikelId
from .enum.auf_abschlagstyp import AufAbschlagstyp
from .enum.auf_abschlagsziel import AufAbschlagsziel
from .enum.ausschreibungsportal import Ausschreibungsportal
from .enum.ausschreibungsstatus import Ausschreibungsstatus
from .enum.ausschreibungstyp import Ausschreibungstyp
from .enum.bdew_artikelnummer import BDEWArtikelnummer
from .enum.bemessungsgroesse import Bemessungsgroesse
from .enum.bilanzierungsmethode import Bilanzierungsmethode
from .enum.bo_typ import BoTyp
from .enum.dienstleistungstyp import Dienstleistungstyp
from .enum.energierichtung import Energierichtung
from .enum.erzeugungsart import Erzeugungsart
from .enum.gasqualitaet import Gasqualitaet
from .enum.gebiettyp import Gebiettyp
from .enum.geraetemerkmal import Geraetemerkmal
from .enum.geraetetyp import Geraetetyp
from .enum.geschaeftspartnerrolle import Geschaeftspartnerrolle
from .enum.gueltigkeitstyp import Gueltigkeitstyp
from .enum.hinweis_thema import HinweisThema
from .enum.kalkulationsmethode import Kalkulationsmethode
from .enum.kontaktart import Kontaktart
from .enum.kostenklasse import Kostenklasse
from .enum.kundengruppe import Kundengruppe
from .enum.kundengruppe_ka import KundengruppeKA
from .enum.kundentyp import Kundentyp
from .enum.landescode import Landescode
from .enum.leistungstyp import Leistungstyp
from .enum.lokationstyp import Lokationstyp
from .enum.marktgebiet import Marktgebiet
from .enum.marktrolle import Marktrolle
from .enum.medium import Medium
from .enum.mengeneinheit import Mengeneinheit
from .enum.mengenoperator import Mengenoperator
from .enum.messart import Messart
from .enum.messgroesse import Messgroesse
from .enum.messpreistyp import Messpreistyp
from .enum.messtechnische_einordnung import MesstechnischeEinordnung
from .enum.messwerterfassung import Messwerterfassung
from .enum.messwertstatus import Messwertstatus
from .enum.messwertstatuszusatz import Messwertstatuszusatz
from .enum.netzebene import Netzebene
from .enum.nn_rechnungsart import NNRechnungsart
from .enum.nn_rechnungstyp import NNRechnungstyp
from .enum.oekolabel import Oekolabel
from .enum.oekozertifikat import Oekozertifikat
from .enum.preisgarantietyp import Preisgarantietyp
from .enum.preismodell import Preismodell
from .enum.preisstatus import Preisstatus
from .enum.preistyp import Preistyp
from .enum.profiltyp import Profiltyp
from .enum.prognosegrundlage import Prognosegrundlage
from .enum.rechnungslegung import Rechnungslegung
from .enum.rechnungsstatus import Rechnungsstatus
from .enum.rechnungstyp import Rechnungstyp
from .enum.regelzone import Regelzone
from .enum.regionskriteriumtyp import Regionskriteriumtyp
from .enum.rollencodetyp import Rollencodetyp
from .enum.rufnummernart import Rufnummernart
from .enum.sparte import Sparte
from .enum.steuerkennzeichen import Steuerkennzeichen
from .enum.tarifart import Tarifart
from .enum.tarifkalkulationsmethode import Tarifkalkulationsmethode
from .enum.tarifmerkmal import Tarifmerkmal
from .enum.tarifregionskriterium import Tarifregionskriterium
from .enum.tariftyp import Tariftyp
from .enum.tarifzeit import Tarifzeit
from .enum.themengebiet import Themengebiet
from .enum.titel import Titel
from .enum.variant import Variant
from .enum.verbrauchsart import Verbrauchsart
from .enum.vertragsart import Vertragsart
from .enum.vertragsform import Vertragsform
from .enum.vertragsstatus import Vertragsstatus
from .enum.voraussetzungen import Voraussetzungen
from .enum.waehrungscode import Waehrungscode
from .enum.waehrungseinheit import Waehrungseinheit
from .enum.wertermittlungsverfahren import Wertermittlungsverfahren
from .enum.zaehlerauspraegung import Zaehlerauspraegung
from .enum.zaehlertyp import Zaehlertyp
from .enum.zeiteinheit import Zeiteinheit



from pydantic import AwareDatetime, BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.zaehlwerk import Zaehlwerk
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.geraetemerkmal import Geraetemerkmal
from ..enum.messwerterfassung import Messwerterfassung
from ..enum.netzebene import Netzebene
from ..enum.sparte import Sparte
from ..enum.tarifart import Tarifart
from ..enum.zaehlerauspraegung import Zaehlerauspraegung
from ..enum.zaehlertyp import Zaehlertyp
from .geschaeftspartner import Geschaeftspartner


class ZaehlerGas(BaseModel):
    """
    Resolve some ambiguity of `Strom` and `Gas`
    """

    model_config = ConfigDict(
        extra="allow",
    )
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.ZAEHLER, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    id: Annotated[str, Field(alias="_id", title=" Id")]
    zaehlernummer: Annotated[str | None, Field(None, title="Zaehlernummer")]
    sparte: Sparte | None = None
    zaehlerauspraegung: Zaehlerauspraegung | None = None
    zaehlertyp: Zaehlertyp
    zaehlwerke: Annotated[list[Zaehlwerk] | None, Field(None, title="Zaehlwerke")]
    tarifart: Tarifart | None = None
    zaehlerkonstante: Annotated[float | str | None, Field(None, title="Zaehlerkonstante")]
    eichung_bis: Annotated[AwareDatetime | None, Field(None, alias="eichungBis", title="Eichungbis")]
    letzte_eichung: Annotated[AwareDatetime | None, Field(None, alias="letzteEichung", title="Letzteeichung")]
    zaehlerhersteller: Geschaeftspartner | None = None
    messwerterfassung: Messwerterfassung
    nachstes_ablesedatum: Annotated[
        AwareDatetime | None, Field(None, alias="nachstesAblesedatum", title="Nachstesablesedatum")
    ]
    aktiver_zeitraum: Annotated[Zeitraum | None, Field(None, alias="aktiverZeitraum")]
    zaehlergroesse: Geraetemerkmal
    druckniveau: Netzebene



from enum import Enum


class Messpreistyp(str, Enum):
    """
    Festlegung, welcher Typ von Messung mit einem Preis belegt wird
    """

    MESSPREIS_G2_5 = "MESSPREIS_G2_5"
    MESSPREIS_G4 = "MESSPREIS_G4"
    MESSPREIS_G6 = "MESSPREIS_G6"
    MESSPREIS_G10 = "MESSPREIS_G10"
    MESSPREIS_G16 = "MESSPREIS_G16"
    MESSPREIS_G25 = "MESSPREIS_G25"
    MESSPREIS_G40 = "MESSPREIS_G40"
    ELEKTRONISCHER_AUFSATZ = "ELEKTRONISCHER_AUFSATZ"
    SMART_METER_MESSPREIS_G2_5 = "SMART_METER_MESSPREIS_G2_5"
    SMART_METER_MESSPREIS_G4 = "SMART_METER_MESSPREIS_G4"
    SMART_METER_MESSPREIS_G6 = "SMART_METER_MESSPREIS_G6"
    SMART_METER_MESSPREIS_G10 = "SMART_METER_MESSPREIS_G10"
    SMART_METER_MESSPREIS_G16 = "SMART_METER_MESSPREIS_G16"
    SMART_METER_MESSPREIS_G25 = "SMART_METER_MESSPREIS_G25"
    SMART_METER_MESSPREIS_G40 = "SMART_METER_MESSPREIS_G40"
    VERRECHNUNGSPREIS_ET_WECHSEL = "VERRECHNUNGSPREIS_ET_WECHSEL"
    VERRECHNUNGSPREIS_ET_DREH = "VERRECHNUNGSPREIS_ET_DREH"
    VERRECHNUNGSPREIS_ZT_WECHSEL = "VERRECHNUNGSPREIS_ZT_WECHSEL"
    VERRECHNUNGSPREIS_ZT_DREH = "VERRECHNUNGSPREIS_ZT_DREH"
    VERRECHNUNGSPREIS_L_ET = "VERRECHNUNGSPREIS_L_ET"
    VERRECHNUNGSPREIS_L_ZT = "VERRECHNUNGSPREIS_L_ZT"
    VERRECHNUNGSPREIS_SM = "VERRECHNUNGSPREIS_SM"
    AUFSCHLAG_WANDLER = "AUFSCHLAG_WANDLER"
    AUFSCHLAG_TARIFSCHALTUNG = "AUFSCHLAG_TARIFSCHALTUNG"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.auf_abschlagstyp import AufAbschlagstyp
from ..enum.waehrungseinheit import Waehrungseinheit


class PositionsAufAbschlag(BaseModel):
    """
    Differenzierung der zu betrachtenden Produkte anhand der preiserhöhenden (Aufschlag)
    bzw. preisvermindernden (Abschlag) Zusatzvereinbarungen,
    die individuell zu einem neuen oder bestehenden Liefervertrag abgeschlossen werden können.
    Es können mehrere Auf-/Abschläge gleichzeitig ausgewählt werden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/PositionsAufAbschlag.svg" type="image/svg+xml"></object>

    .. HINT::
        `PositionsAufAbschlag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/PositionsAufAbschlag.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    auf_abschlagstyp: Annotated[AufAbschlagstyp | None, Field(None, alias="aufAbschlagstyp")]
    auf_abschlagswaehrung: Annotated[Waehrungseinheit | None, Field(None, alias="aufAbschlagswaehrung")]
    auf_abschlagswert: Annotated[float | str | None, Field(None, alias="aufAbschlagswert", title="Aufabschlagswert")]
    beschreibung: Annotated[str | None, Field(None, title="Beschreibung")]
    bezeichnung: Annotated[str | None, Field(None, title="Bezeichnung")]



from enum import Enum


class Regionskriteriumtyp(str, Enum):
    """
    Klassifizierung der Kriterien für eine regionale Eingrenzung.
    """

    BUNDESLANDKENNZIFFER = "BUNDESLANDKENNZIFFER"
    BUNDESLAND_NAME = "BUNDESLAND_NAME"
    MARKTGEBIET_NUMMER = "MARKTGEBIET_NUMMER"
    MARKTGEBIET_NAME = "MARKTGEBIET_NAME"
    REGELGEBIET_NUMMER = "REGELGEBIET_NUMMER"
    REGELGEBIET_NAME = "REGELGEBIET_NAME"
    NETZ_STROM = "NETZ_STROM"
    NETZ_GAS = "NETZ_GAS"
    NETZBETREIBER_NUMMER_STROM = "NETZBETREIBER_NUMMER_STROM"
    NETZBETREIBER_NUMMER_GAS = "NETZBETREIBER_NUMMER_GAS"
    NETZBETREIBER_NAME_STROM = "NETZBETREIBER_NAME_STROM"
    NETZBETREIBER_NAME_GAS = "NETZBETREIBER_NAME_GAS"
    BILANZIERUNGS_GEBIET_NUMMER = "BILANZIERUNGS_GEBIET_NUMMER"
    MSB_NUMMER = "MSB_NUMMER"
    MSB_NAME = "MSB_NAME"
    VERSORGER_NUMMER = "VERSORGER_NUMMER"
    VERSORGER_NAME = "VERSORGER_NAME"
    GRUNDVERSORGER_NUMMER_STROM = "GRUNDVERSORGER_NUMMER_STROM"
    GRUNDVERSORGER_NAME_STROM = "GRUNDVERSORGER_NAME_STROM"
    GRUNDVERSORGER_NUMMER_GAS = "GRUNDVERSORGER_NUMMER_GAS"
    GRUNDVERSORGER_NAME_GAS = "GRUNDVERSORGER_NAME_GAS"
    KREIS_NAME = "KREIS_NAME"
    KREISKENNZIFFER = "KREISKENNZIFFER"
    GEMEINDE_NAME = "GEMEINDE_NAME"
    GEMEINDEKENNZIFFER = "GEMEINDEKENNZIFFER"
    POSTLEITZAHL = "POSTLEITZAHL"
    ORT = "ORT"
    POSTORT = "POSTORT"
    EINWOHNERZAHL_GEMEINDE = "EINWOHNERZAHL_GEMEINDE"
    EINWOHNERZAHL_ORT = "EINWOHNERZAHL_ORT"
    KM_UMKREIS = "KM_UMKREIS"
    BUNDESWEIT = "BUNDESWEIT"
    PLZ_BEREICH = "PLZ_BEREICH"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.mengeneinheit import Mengeneinheit


class Zaehlpunkt(BaseModel):
    """
    The zaehlpunkt object was created during a migration project.
    It contains attributes needed for metering mapping.
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    periodenverbrauch_vorhersage: Annotated[
        float | str, Field(alias="periodenverbrauchVorhersage", title="Periodenverbrauchvorhersage")
    ]
    einheit_vorhersage: Annotated[Mengeneinheit | None, Field(Mengeneinheit.KWH, alias="einheitVorhersage")]
    zeitreihentyp: Annotated[str | None, Field("Z21", title="Zeitreihentyp")]
    kunden_wert: Annotated[float | str | None, Field(alias="kundenWert", title="Kundenwert")]
    einheit_kunde: Annotated[Mengeneinheit | None, Field(None, alias="einheitKunde")]
    grundzustaendiger: Annotated[bool | None, Field(True, title="Grundzustaendiger")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.zeitreihenwert import Zeitreihenwert
from ..enum.bo_typ import BoTyp
from ..enum.medium import Medium
from ..enum.mengeneinheit import Mengeneinheit
from ..enum.messart import Messart
from ..enum.messgroesse import Messgroesse
from ..enum.wertermittlungsverfahren import Wertermittlungsverfahren


class Zeitreihe(BaseModel):
    """
    Abbildung einer allgemeinen Zeitreihe mit einem Wertvektor.
    Die Werte können mit wahlfreier zeitlicher Distanz im Vektor abgelegt sein.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Zeitreihe.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitreihe JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Zeitreihe.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    beschreibung: Annotated[str | None, Field(None, title="Beschreibung")]
    bezeichnung: Annotated[str | None, Field(None, title="Bezeichnung")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.ZEITREIHE, alias="boTyp")]
    einheit: Mengeneinheit | None = None
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    medium: Medium | None = None
    messart: Messart | None = None
    messgroesse: Messgroesse | None = None
    version: Annotated[str | None, Field(None, title="Version")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    werte: Annotated[list[Zeitreihenwert] | None, Field(None, title="Werte")]
    wertherkunft: Wertermittlungsverfahren | None = None



from pydantic import AwareDatetime, BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from .zeitreihenwertkompakt import Zeitreihenwertkompakt


class Tagesvektor(BaseModel):
    """
    Abbildung eines Tagesvektors eines beliebigen äquidistanten Zeitrasters

    .. raw:: html

        <object data="../_static/images/bo4e/com/Tagesvektor.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tagesvektor JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Tagesvektor.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    tag: Annotated[AwareDatetime | None, Field(None, title="Tag")]
    werte: Annotated[list[Zeitreihenwertkompakt] | None, Field(None, title="Werte")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from .betrag import Betrag
from .menge import Menge
from .preis import Preis


class Angebotsposition(BaseModel):
    """
    Unterhalb von Angebotsteilen sind die Angebotspositionen eingebunden.
    Hier werden die angebotenen Bestandteile einzeln aufgeführt. Beispiel:
    Positionsmenge: 4000 kWh
    Positionspreis: 24,56 ct/kWh
    Positionskosten: 982,40 EUR

    .. raw:: html

        <object data="../_static/images/bo4e/com/Angebotsposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Angebotsposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Angebotsposition.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
    )
    id: Annotated[str, Field(alias="_id", title=" Id")]
    positionsbezeichnung: Annotated[str | None, Field(None, title="Positionsbezeichnung")]
    positionskosten: Betrag | None = None
    positionsmenge: Menge | None = None
    positionspreis: Preis | None = None



from enum import Enum


class Zeiteinheit(str, Enum):
    """
    Auflistung möglicher Einheiten zur Verwendung in zeitbezogenen Angaben.
    """

    SEKUNDE = "SEKUNDE"
    MINUTE = "MINUTE"
    STUNDE = "STUNDE"
    VIERTEL_STUNDE = "VIERTEL_STUNDE"
    TAG = "TAG"
    WOCHE = "WOCHE"
    MONAT = "MONAT"
    QUARTAL = "QUARTAL"
    HALBJAHR = "HALBJAHR"
    JAHR = "JAHR"
